__version__ = "0.0.2"
__author__ = "osoken"
__description__ = "Swiss Army Knife-like toolbox for data processing"
__email__ = "osoken.devel@outlook.jp"
__package_name__ = "jittok"


from .core import Callable

__all__ = ["Callable"]
